/*
*
* Description: library for reading caltrack MODIS L1 data records
*
* History :
*   v0.0.0 2008/02/25 : creation
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) Centre de Gestion et de Traitement de Donnes (CGTD) ICARE 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#ifndef CALTRACK_READER_MODIS_L1_H
#define CALTRACK_READER_MODIS_L1_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_MODIS_L1 stores the variables issued of the MODIS L1 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    /** Index [i_row,icol] in the MODIS FOV of the coincident Pixel in the input product */
    int16  input_pixel_index                  [2];
    /** Index of the file in the source files list */
    int16  input_file_index;
    /** Name of the file where has been found the coincidence, format YYYY_MM_DD/filename where YYY, MM and DD are the year, month, day of the granule */
    char   input_file                   [MAX_NC_NAME];
    /** Earth View 250M Aggregated 1km Reflective Solar Bands Radiances of channel n1 (0.645m) */
    float64 ev_250_aggr1km_refsb_band1_radiance;
    float64 ev_250_aggr1km_refsb_band1_reflectance;
    /** Earth View 250M Aggregated 1km Reflective Solar Bands Radiances of channel n2 (0.865m) */
    float64 ev_250_aggr1km_refsb_band2_radiance;
    float64 ev_250_aggr1km_refsb_band2_reflectance;
    /** Earth View 500M Aggregated 1km Reflective Solar Bands Radiances of channel n3 (0.469m) */
    float64 ev_500_aggr1km_refsb_band3_radiance;
    float64 ev_500_aggr1km_refsb_band3_reflectance;
    /** Earth View 500M Aggregated 1km Reflective Solar Bands Radiances of channel n4 (0.555m) */
    float64 ev_500_aggr1km_refsb_band4_radiance;
    float64 ev_500_aggr1km_refsb_band4_reflectance;
    /** Earth View 500M Aggregated 1km Reflective Solar Bands Radiances of channel n5 (1.240m) */
    float64 ev_500_aggr1km_refsb_band5_radiance;
    float64 ev_500_aggr1km_refsb_band5_reflectance;
    /** Earth View 500M Aggregated 1km Reflective Solar Bands Radiances of channel n6 (1.640m) */
    float64 ev_500_aggr1km_refsb_band6_radiance;
    float64 ev_500_aggr1km_refsb_band6_reflectance;
    /** Earth View 500M Aggregated 1km Reflective Solar Bands Radiances of channel n7 (2.130m) */
    float64 ev_500_aggr1km_refsb_band7_radiance;
    float64 ev_500_aggr1km_refsb_band7_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n8 (0.4125m) */
    float64 ev_1km_refsb_band8_radiance;
    float64 ev_1km_refsb_band8_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n9 (0.443m) */
    float64 ev_1km_refsb_band9_radiance;
    float64 ev_1km_refsb_band9_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n10 (0.488m) */
    float64 ev_1km_refsb_band10_radiance;
    float64 ev_1km_refsb_band10_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n11 (0.541m) */
    float64 ev_1km_refsb_band11_radiance;
    float64 ev_1km_refsb_band11_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n12 (0.551m) */
    float64 ev_1km_refsb_band12_radiance;
    float64 ev_1km_refsb_band12_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n13 Low (0.667m) */
    float64 ev_1km_refsb_band13lo_radiance;
    float64 ev_1km_refsb_band13lo_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n13 High (0.667m) */
    float64 ev_1km_refsb_band13hi_radiance;
    float64 ev_1km_refsb_band13hi_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n14 Low (0.678m) */
    float64 ev_1km_refsb_band14lo_radiance;
    float64 ev_1km_refsb_band14lo_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n14 High (0.678m) */
    float64 ev_1km_refsb_band14hi_radiance;
    float64 ev_1km_refsb_band14hi_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n15 (0.748m) */
    float64 ev_1km_refsb_band15_radiance;
    float64 ev_1km_refsb_band15_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n16 (1.3005m) */
    float64 ev_1km_refsb_band16_radiance;
    float64 ev_1km_refsb_band16_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n17 (0.905m) */
    float64 ev_1km_refsb_band17_radiance;
    float64 ev_1km_refsb_band17_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n18 (0.936m) */
    float64 ev_1km_refsb_band18_radiance;
    float64 ev_1km_refsb_band18_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n19 (0.940m) */
    float64 ev_1km_refsb_band19_radiance;
    float64 ev_1km_refsb_band19_reflectance;
    /** Earth View 1KM Reflective Solar Bands of channel n26 (1.375m) */
    float64 ev_1km_refsb_band26_radiance;
    float64 ev_1km_refsb_band26_reflectance;
    /** Earth View 1KM Emissive Bands of channel n20 (3.750m) */
    float64 ev_1km_emissive_band20_radiance;
    /** Earth View 1KM Emissive Bands of channel n21 (3.959m) */
    float64 ev_1km_emissive_band21_radiance;
    /** Earth View 1KM Emissive Bands of channel n22 (3.959m) */
    float64 ev_1km_emissive_band22_radiance;
    /** Earth View 1KM Emissive Bands of channel n23 (4.050m) */
    float64 ev_1km_emissive_band23_radiance;
    /** Earth View 1KM Emissive Bands of channel n24 (4.465m) */
    float64 ev_1km_emissive_band24_radiance;
    /** Earth View 1KM Emissive Bands of channel n25 (4.515m) */
    float64 ev_1km_emissive_band25_radiance;
    /** Earth View 1KM Emissive Bands of channel n27 (6.715m) */
    float64 ev_1km_emissive_band27_radiance;
    /** Earth View 1KM Emissive Bands of channel n28 (7.325m) */
    float64 ev_1km_emissive_band28_radiance;
    /** Earth View 1KM Emissive Bands of channel n29 (8.550m) */
    float64 ev_1km_emissive_band29_radiance;
    /** Earth View 1KM Emissive Bands of channel n30 (9.730m) */
    float64 ev_1km_emissive_band30_radiance;
    /** Earth View 1KM Emissive Bands of channel n31 (11.030m) */
    float64 ev_1km_emissive_band31_radiance;
    /** Earth View 1KM Emissive Bands of channel n32 (12.020m) */
    float64 ev_1km_emissive_band32_radiance;
    /** Earth View 1KM Emissive Bands of channel n33 (13.335m) */
    float64 ev_1km_emissive_band33_radiance;
    /** Earth View 1KM Emissive Bands of channel n34 (13.635m) */
    float64 ev_1km_emissive_band34_radiance;
    /** Earth View 1KM Emissive Bands of channel n35 (13.935m) */
    float64 ev_1km_emissive_band35_radiance;
    /** Earth View 1KM Emissive Bands of channel n36 (14.235m) */
    float64 ev_1km_emissive_band36_radiance;

} caltrack_record_MODIS_L1;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_MODIS_L1 ( const char* filename, const int i_rec, caltrack_record_MODIS_L1 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_MODIS_L1 ( const caltrack_record_MODIS_L1 const *record );

/**
 * @brief retrieve the scaling factors of the given channel
 * @param sd_id [IN] an SD id as returned by the open_hdf_file function
 * @param sds_name [IN] name of the sds to read
 * @param radiance_scale [OUT] radiance scaling factor
 * @param radiance_offset [OUT] radiance scaling offset
 * @param reflectance_scale [OUT] reflectance scaling factor
 * @param reflectance_offset [OUT] reflectance scaling offset
 * @return 0 if successfull, -1 in case of error
 */
int32 get_myd021km_band_scaling_factor(const int32 sd_id, const char* sds_name,
                                             float64 * radiance_scale, float64 * radiance_offset,
                                             float64 * reflectance_scale, float64 * reflectance_offset);

/**
 * @brief read the value of the given MODIS channel for one record
 * @param sd_id [IN] an SD id as returned by the open_hdf_file function
 * @param sds_name [IN] name of the sds to read
 * @param i_rec [IN] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param radiance [OUT] the scaled radiance
 * @param reflectance [OUT] the scaled reflectance (if consistent. Set it to NULL for reading Emissive bands)
 */
int32 read_band_data (const int32 sd_id, const char* sds_name, const int32 i_rec,
                      float64 * radiance, float64 * reflectance);

#endif
