//
// C++ Interface: caltrack_reader_modis_myd05
//
// Description: 
//
//
// Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef CALTRACK_READER_MODIS_MYD05_H
#define CALTRACK_READER_MODIS_MYD05_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_MODIS_MYD05 stores the variables issued of the MODIS MYD05 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    int16  input_pixel_index            [2];
    int16  input_file_index;
    char   input_file                   [MAX_NC_NAME];
    float64 water_vapor_near_infrared;
    float64 water_vapor_infrared;
    float64 water_vapor_correction_factors;
    int8    quality_assurance_infrared[5];
    int8    quality_assurance_near_infrared;
    int8    cloud_mask;
} caltrack_record_MODIS_MYD05;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_MODIS_MYD05 ( const char* filename, const int i_rec, caltrack_record_MODIS_MYD05 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_MODIS_MYD05 ( const caltrack_record_MODIS_MYD05 const *record );

#endif
