//
// C++ Interface: caltrack_reader_modis_myd06
//
// Description: 
//
//
// Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef CALTRACK_READER_MODIS_MYD06_H
#define CALTRACK_READER_MODIS_MYD06_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_MODIS_MYD06 stores the variables issued of the MODIS MYD06 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    /** Index [i_row,icol] in the MODIS FOV of the coincident Pixel in the input product */
    int16  input_pixel_index                  [2];
    /** Index of the file in the source files list */
    int16  input_file_index;
    /** Name of the file where has been found the coincidence, format YYYY_MM_DD/filename where YYY, MM and DD are the year, month, day of the granule */
    char   input_file                   [MAX_NC_NAME];
    float64 brightness_temperature_band_29;
    float64 brightness_temperature_band_31;
    float64 brightness_temperature_band_32;
    float64 brightness_temperature_band_33;
    float64 brightness_temperature_band_34;
    float64 brightness_temperature_band_35;
    float64 brightness_temperature_band_36;
    int8    number_clear_pixel;
    int8    number_cloudy_pixel;
    int8    number_indetermined_pixel;
    float64 cloud_optical_thickness;
    float64 cloud_water_path;
    float64 cloud_top_pressure;
    float64 cloud_effective_radius;
    int16   cloud_phase_optical_properties;
    int16   cloud_multi_layer_flag;
    float64 cloud_fraction;
    int8    cloud_mask_5km;
    int8    cloud_mask_1km        [ 2];
    int8    quality_assurance_5km [10];
    int8    quality_assurance_1km [ 5];
} caltrack_record_MODIS_MYD06;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_MODIS_MYD06 ( const char* filename, const int i_rec, caltrack_record_MODIS_MYD06 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_MODIS_MYD06 ( const caltrack_record_MODIS_MYD06 const *record );

#endif
