/*
* C Interface: caltrack_reader_omi_cldrr
*
* Description:
*   Set the structure and the functions to read a record of a OMI CLDRR CALTRACK_UNIT product
*
* Author:
*   CGTD-ICARE/UDEV Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2009
*
* License :
*   This file must be used under the terms of the CeCILL.
*   This source file is licensed as described in the file COPYING, which
*   you should have received as part of this distribution.  The terms
*   are also available at
*   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#ifndef CALTRACK_READER_OMI_CLDRR_H
#define CALTRACK_READER_OMI_CLDRR_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_OMI_CLDRR stores the variables issued of the OMI CLDRR CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    float64 cloud_fraction;
    float64 rad_cloud_fraction;
    float64 cloud_pressure;
} caltrack_record_OMI_CLDRR;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_OMI_CLDRR ( const char* filename, const int i_rec, caltrack_record_OMI_CLDRR *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_OMI_CLDRR ( const caltrack_record_OMI_CLDRR const *record );

#endif
