//
// C++ Interface: caltrack_reader_parasol_ls2
//
// Description: 
//
//
// Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef CALTRACK_READER_PARASOL_LS2_H
#define CALTRACK_READER_PARASOL_LS2_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_PARASOL_LS2 stores the variables issued of the PARASOL LS2 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    int16   input_index[2];
    uint8   land_water_indicator;
    uint16  pixel_confidence_data;
    float64 aerosol_od_865;
    float64 aerosol_model_refractive_index;
    float64 aerosol_model_angstrom_coefficient;
    float64 aerosol_index;
    float64 fixed_model_od_865;
    float64 quality_index_viewing_geometry;
    float64 quality_index_polarized_reflectance_fit;
} caltrack_record_PARASOL_LS2;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_PARASOL_LS2 ( const char* filename, const int i_rec, caltrack_record_PARASOL_LS2 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_PARASOL_LS2 ( const caltrack_record_PARASOL_LS2 const *record );

#endif
