//
// C++ Interface: caltrack_reader_parasol_oc2
//
// Description: 
//
//
// Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef CALTRACK_READER_PARASOL_OC2_H
#define CALTRACK_READER_PARASOL_OC2_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_PARASOL_OC2 stores the variables issued of the PARASOL OC2 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    int16   input_index[2];
    uint32  pixel_confidence_data;
    float64 fit_quality;
    float64 solar_zenith_angle;
    float64 aerosol_od_865;
    float64 aerosol_od_670;
    float64 aerosol_angstrom_coefficient;
    float64 single_scattering_albedo_865;
    float64 aerosol_scattering_asymmetry_factor;
    float64 aerosol_index;
    float64 aerosol_effective_radius;
    float64 fine_mode_effective_radius;
    float64 large_mode_effective_radius;
    float64 fine_mode_od_865;
    float64 fine_mode_od_670;
    float64 fine_mode_angstrom_exponent;
    float64 spherical_coarse_mode_od_865;
    float64 non_spherical_coarse_mode_od_865;
    float64 relative_contrib_non_spherical_part_coarse_mode;
    float64 refractive_index_fine_mode;
    float64 refractive_index_coarse_mode;
    float64 log_backscattering_coefficient_565;
    float64 log_backscattering_coefficient_1020;
} caltrack_record_PARASOL_OC2;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_PARASOL_OC2 ( const char* filename, const int i_rec, caltrack_record_PARASOL_OC2 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_PARASOL_OC2 ( const caltrack_record_PARASOL_OC2 const *record );

#endif
