/*
* C Interface: caltrack_reader_parasol_rb2
*
* Description:
*   Set the structure and the functions to read a record of a PARASOL RB2 CALTRACK_UNIT product
*
* Author:
*   CGTD-ICARE/UDEV Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* License :
*   This file must be used under the terms of the CeCILL.
*   This source file is licensed as described in the file COPYING, which
*   you should have received as part of this distribution.  The terms
*   are also available at
*   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#ifndef CALTRACK_READER_PARASOL_RB2_H
#define CALTRACK_READER_PARASOL_RB2_H

#include "caltrack_reader_common.h"

/**
 * @struct caltrack_record_PARASOL_RB2 stores the variables issued of the PARASOL RB2 CALTRACK files
 */
typedef struct {
    float32 lat;
    float32 lon;
    float64 time;
    int16   input_index[2];
    float64 cloud_cover;
    float64 directional_cloud_cover [NB_PARASOL_DIRECTION];
    float64 cloud_rayleigh_pressure;
    float64 cloud_rayleigh_pressure_angular_stddev;
    float64 cloud_oxygen_pressure;
    float64 cloud_oxygen_pressure_angular_stddev;
    float64 cloud_optical_thickness;
    float64 cloud_optical_thickness_stddev;
    uint8   cloud_phase;
    float64 water_vapor_column;
    float64 mean_albedo;
    float64 albedo_spatial_stddev;
    float64 albedo_angular_stddev;
    uint8   surface_type_index;
    float64 cos_solar_zenith_angle;
    float64 scene_albedo_angular_stddev;
    float64 optical_thickness_homogeneity_coef;

} caltrack_record_PARASOL_RB2;

/**
 * @brief read the record at index "i_rec" and stores the values in "record"
 * @param filename full path to the file to read
 * @param i_rec [input] index of the record (ie CALIOP profile index) to read. Range [0,nb_record-1]
 * @param record [output] stores the read values
 * @return 0 if all works well, -1 else
 */
int read_caltrack_record_PARASOL_RB2 ( const char* filename, const int i_rec, caltrack_record_PARASOL_RB2 *record );

/**
 * @brief print out the values of the given record
 * @param record the record to print
 */
void print_caltrack_record_PARASOL_RB2 ( const caltrack_record_PARASOL_RB2 const *record );

#endif
