/*
*  C Implementation: caltrack_reader_omi_cldrr
*
* Description:
*   Set the structure and the functions to read a record of a OMI CLDRR CALTRACK_UNIT product
*
* Author:
*   CGTD-ICARE/UDEV Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2009
*
* License :
*   This file must be used under the terms of the CeCILL.
*   This source file is licensed as described in the file COPYING, which
*   you should have received as part of this distribution.  The terms
*   are also available at
*   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#include "caltrack_reader_omi_cldrr.h"

int read_caltrack_record_OMI_CLDRR( const char* filename, const int i_rec, caltrack_record_OMI_CLDRR *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* name of an SDS */
    char sds_name[MAX_NC_NAME];
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        return FAIL;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"omi_cldrr_cloud_fraction");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_fraction, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"omi_cldrr_rad_cloud_fraction");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->rad_cloud_fraction, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"omi_cldrr_cloud_pressure");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_pressure, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
};

void print_caltrack_record_OMI_CLDRR ( const caltrack_record_OMI_CLDRR const * record ) {
    printf( "Latitude %f\n",                                    record->lat );
    printf( "Longitude %f\n",                                   record->lon );
    printf( "Time %e\n",                                        record->time );
    printf( "Cloud Fraction %e\n",                              record->cloud_fraction );
    printf( "Radiative Cloud Fraction %e\n",                    record->rad_cloud_fraction );
    printf( "Cloud Pressure %e\n",                              record->cloud_pressure );
}
