/*
*  C Implementation: caltrack_reader_caliop_05kmalay
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader.h"

int read_caltrack_record_CALIOP_05KMALAY( const char* filename, const int i_rec, caltrack_record_CALIOP_05KMALAY *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,NB_AEROSOL_LAYER_CALIOP};
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Number_Layers_Found");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->number_layers_found, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Layer_Top_Altitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->layer_top_altitude, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Layer_Base_Altitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->layer_base_altitude, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Feature_Classification_Flags");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->feature_classification_flags, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Layer_Top_Temperature");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->layer_top_temperature, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Layer_Base_Temperature");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->layer_base_temperature, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Layer_Top_Pressure");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->layer_top_pressure, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Layer_Base_Pressure");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->layer_base_pressure, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_LID_L2_05KMALAY_Feature_Optical_Depth_532");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->feature_optical_depth_532, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_CALIOP_05KMALAY( const caltrack_record_CALIOP_05KMALAY const * record ) {
    printf( "Latitude %f\n"          ,                          record->lat );
    printf( "Longitude %f\n"         ,                          record->lon );
    printf( "Time %e\n"              ,                          record->time );
    printf( "Number Layers Found %d\n"  ,                       record->number_layers_found);
    printf( "Layer Top Altitude [%f\t%f\t%f\t%f\t...]\n"  ,     record->layer_top_altitude[0],
                                                                record->layer_top_altitude[1],
                                                                record->layer_top_altitude[2],
                                                                record->layer_top_altitude[3] );
    printf( "Layer Base Altitude [%f\t%f\t%f\t%f\t...]\n"  ,    record->layer_base_altitude[0],
                                                                record->layer_base_altitude[1],
                                                                record->layer_base_altitude[2],
                                                                record->layer_base_altitude[3] );
    printf( "Layer Top Temperature [%f\t%f\t%f\t%f\t...]\n"  ,  record->layer_top_temperature[0],
                                                                record->layer_top_temperature[1],
                                                                record->layer_top_temperature[2],
                                                                record->layer_top_temperature[3] );
    printf( "Layer Base Temperature [%f\t%f\t%f\t%f\t...]\n"  , record->layer_base_temperature[0],
                                                                record->layer_base_temperature[1],
                                                                record->layer_base_temperature[2],
                                                                record->layer_base_temperature[3] );
    printf( "Layer Top Pressure [%f\t%f\t%f\t%f\t...]\n"  ,     record->layer_top_pressure[0],
                                                                record->layer_top_pressure[1],
                                                                record->layer_top_pressure[2],
                                                                record->layer_top_pressure[3] );
    printf( "Layer Base Pressure [%f\t%f\t%f\t%f\t...]\n"  ,    record->layer_base_pressure[0],
                                                                record->layer_base_pressure[1],
                                                                record->layer_base_pressure[2],
                                                                record->layer_base_pressure[3] );
    printf( "Feature Classification Flags [%x\t%x\t%x\t%x\t...]\n",
                                                                record->feature_classification_flags[0],
                                                                record->feature_classification_flags[1],
                                                                record->feature_classification_flags[2],
                                                                record->feature_classification_flags[3] );
    printf( "Feature Optical Depth 532nm [%f\t%f\t%f\t%f\t...]\n"  ,    record->feature_optical_depth_532[0],
                                                                record->feature_optical_depth_532[1],
                                                                record->feature_optical_depth_532[2],
                                                                record->feature_optical_depth_532[3] );
}
