/*
*  C Implementation: caltrack_reader_calipso_iir_l2_track
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader.h"

int read_caltrack_record_CALIPSO_IIR_L2_TRACK( const char* filename, const int i_rec, caltrack_record_CALIPSO_IIR_L2_TRACK *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_IIR_L2_Brightness_Temperature_12_05");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->brightness_temperature_12_05, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_IIR_L2_Brightness_Temperature_10_60");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->brightness_temperature_10_60, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_IIR_L2_Brightness_Temperature_08_65");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->brightness_temperature_08_65, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_IIR_L2_Effective_Particle_Size");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->effective_particle_size, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_IIR_L2_Particle_Shape_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->particle_shape_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CAL_IIR_L2_Ice_Water_Path");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->ice_water_path, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_CALIPSO_IIR_L2_TRACK( const caltrack_record_CALIPSO_IIR_L2_TRACK const * record ) {
    printf( "Latitude %f\n"          ,                          record->lat );
    printf( "Longitude %f\n"         ,                          record->lon );
    printf( "Time %e\n"              ,                          record->time );
    printf( "Brightness Temperature 12.05m %f\n"  ,            record->brightness_temperature_12_05);
    printf( "Brightness Temperature 10.60m %f\n"  ,            record->brightness_temperature_10_60);
    printf( "Brightness Temperature  8.65m %f\n"  ,            record->brightness_temperature_08_65);
    printf( "Effective Particle Size %f\n"  ,                   record->effective_particle_size);
    printf( "Particle Shape Index %d\n"  ,                      record->particle_shape_index);
    printf( "Ice Water Path %f\n"  ,                            record->ice_water_path);
}
