/*
*  C Implementation: caltrack_reader_cloudsat_2b_cldclass
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/
/*
*  C Implementation: caltrack_reader_cloudsat_2b_geoprof
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader_cloudsat_2b_cldclass.h"

int read_caltrack_record_CLOUDSAT_2B_CLDCLASS( const char * filename, const int i_rec, caltrack_record_CLOUDSAT_2B_CLDCLASS * record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* name of the product */
    char * product_name = "CLOUDSAT 2B-CLDCLASS"; // suite  un bug dans caltrack <0.6.1,  corriger dans les versions suivantes
    /* generic pointer */
    void* p_data = NULL;
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,NB_CLOUDSAT_VERTICAL_BINS};

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CS_2B_CLDCLASS_Input_File_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_file_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    status = get_input_file_by_id (sd_id,product_name,record->input_file_index,record->input_file);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Fail to read the input files of product %s in %s\n",__FILE__,__LINE__,product_name,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CS_2B_CLDCLASS_Input_Profile_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_profile_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CS_2B_CLDCLASS_Height");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->height, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"CS_2B_CLDCLASS_cloud_scenario");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->cloud_scenario, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_CLOUDSAT_2B_CLDCLASS ( const caltrack_record_CLOUDSAT_2B_CLDCLASS const *record ) {
    printf( "Latitude %f\n"          ,  record->lat );
    printf( "Longitude %f\n"         ,  record->lon );
    printf( "Time %e\n"              ,  record->time );
    printf( "Input File %s\n"        ,  record->input_file );
    printf( "Input File index %d\n"  ,  record->input_file_index);
    printf( "Input Profile index %d\n", record->input_profile_index );
    printf( "Height=[%d\t%d\t...\t%d %d]\n",              record->height[0],
                                                          record->height[1],
                                                          record->height[NB_CLOUDSAT_VERTICAL_BINS-2],
                                                          record->height[NB_CLOUDSAT_VERTICAL_BINS-1]);
    printf( "CPR Cloud Scenario=[%d\t%d\t...\t%d %d]\n",  record->cloud_scenario[0],
                                                          record->cloud_scenario[1],
                                                          record->cloud_scenario[NB_CLOUDSAT_VERTICAL_BINS-2],
                                                          record->cloud_scenario[NB_CLOUDSAT_VERTICAL_BINS-1]);

}

