/*
*  C Implementation: caltrack_reader_modis_myd04
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader_modis_myd04.h"

int read_caltrack_record_MODIS_MYD04( const char* filename, const int i_rec, caltrack_record_MODIS_MYD04 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* name of the product */
    char * product_name = "MYD04";
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,-1};
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD04_Input_File_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_file_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    status = get_input_file_by_id (sd_id,product_name,record->input_file_index,record->input_file);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Fail to read the input files of product %s in %s\n",__FILE__,__LINE__,product_name,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD04_Input_Pixel_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->input_pixel_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD04_Optical_Depth_Land_And_Ocean");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->optical_depth_land_and_ocean, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD04_Optical_Depth_Ratio_Small_Land_And_Ocean");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name,&record->optical_depth_ratio_small_land_and_ocean, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD04_Effective_Optical_Depth_Best_Ocean");
    edges_2D[1]=7;
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->effective_optical_depth_best_ocean, start_2D, NULL, edges_2D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD04_Angstrom_Exponent_1_Ocean");
    edges_2D[1]=2;
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->angstrom_exponent_1_ocean, start_2D, NULL, edges_2D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_MODIS_MYD04( const caltrack_record_MODIS_MYD04 const * record ) {
    printf( "Latitude %f\n"          ,                          record->lat );
    printf( "Longitude %f\n"         ,                          record->lon );
    printf( "Time %e\n"              ,                          record->time );
    printf( "Input index [%d,%d]\n"  ,                          record->input_pixel_index[0], record->input_pixel_index[1] );
    printf( "Input File index %d\n"  ,                          record->input_file_index);
    printf( "Input File %s\n"        ,                          record->input_file );

    printf( "Optical Depth Land and Ocean %e\n"              ,  record->optical_depth_land_and_ocean );
    printf( "Optical Depth Ratio Small Land and Ocean %e\n"  ,  record->optical_depth_ratio_small_land_and_ocean );
    printf( "Effective Optical Depth Best Ocean [%e\t%e\t%e\t%e\t%e\t%e\t%e]\n",
                                                                record->effective_optical_depth_best_ocean[0],
                                                                record->effective_optical_depth_best_ocean[1],
                                                                record->effective_optical_depth_best_ocean[2],
                                                                record->effective_optical_depth_best_ocean[3],
                                                                record->effective_optical_depth_best_ocean[4],
                                                                record->effective_optical_depth_best_ocean[5],
                                                                record->effective_optical_depth_best_ocean[6] );
    printf( "Angstrm Exponent 1 Ocean [%e\t%e]\n",             record->angstrom_exponent_1_ocean[0],
                                                                record->angstrom_exponent_1_ocean[1] );
}
