/*
*  C Implementation: caltrack_reader_modis_myd05
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader_modis_myd05.h"

int read_caltrack_record_MODIS_MYD05( const char* filename, const int i_rec, caltrack_record_MODIS_MYD05 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* name of the product */
    char * product_name = "MYD05";
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,-1};
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Input_Pixel_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, (void*)record->input_pixel_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Input_File_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_file_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    status = get_input_file_by_id (sd_id,product_name,record->input_file_index,record->input_file);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Fail to read the input files of product %s in %s\n",__FILE__,__LINE__,product_name,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Water_Vapor_Near_Infrared");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->water_vapor_near_infrared, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Water_Vapor_Infrared");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->water_vapor_infrared, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Water_Vapor_Correction_Factors");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->water_vapor_correction_factors, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Quality_Assurance_Infrared");
    edges_2D[1]=5;
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->quality_assurance_infrared, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Quality_Assurance_Near_Infrared");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->quality_assurance_near_infrared, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD05_Cloud_Mask");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, (void*)&record->cloud_mask, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_MODIS_MYD05( const caltrack_record_MODIS_MYD05 const * record ) {
    printf( "Latitude %f\n"          ,                          record->lat );
    printf( "Longitude %f\n"         ,                          record->lon );
    printf( "Time %e\n"              ,                          record->time );
    printf( "Input index [%d,%d]\n"  ,                          record->input_pixel_index[0], record->input_pixel_index[1] );
    printf( "Input File index %d\n"  ,                          record->input_file_index);
    printf( "Input File %s\n"        ,                          record->input_file );
    printf( "Water Vapor Near Infrared %e\n",                   record->water_vapor_near_infrared );
    printf( "Water Vapor Infrared %e\n",                        record->water_vapor_infrared );
    printf( "Water Vapor Correction Factors %e\n",              record->water_vapor_correction_factors );
    printf( "Quality Assurance Infrared [%i\t%i\t%i\t%i\t%i]\n",
                                                                record->quality_assurance_infrared[0],
                                                                record->quality_assurance_infrared[1],
                                                                record->quality_assurance_infrared[2],
                                                                record->quality_assurance_infrared[3],
                                                                record->quality_assurance_infrared[4] );
    printf( "Quality Assurance Near Infrared %i\n",             record->quality_assurance_near_infrared );
    printf( "Cloud Mask %i\n",                                  record->cloud_mask );
}
