/*
*  C Implementation: caltrack_reader_modis_myd06
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader_modis_myd06.h"

int read_caltrack_record_MODIS_MYD06( const char* filename, const int i_rec, caltrack_record_MODIS_MYD06 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* Name of a SDS to read */
    char sds_name[MAX_NC_NAME];
    /* name of the product */
    char * product_name = "MYD06";
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,-1};
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Input_File_Index");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_file_index, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    status = get_input_file_by_id (sd_id,product_name,record->input_file_index,record->input_file);
    if (status == FAIL) {
        fprintf(stderr,"%s[L%d] Fail to read the input files of product %s in %s\n",__FILE__,__LINE__,product_name,filename);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Input_Pixel_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->input_file_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_29");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_29, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_31");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_31, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_32");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_32, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_33");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_33, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_34");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_34, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_35");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_35, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Brightness_Temperature_Band_36");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->brightness_temperature_band_36, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Number_Clear_Pixel");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->number_clear_pixel, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Number_Cloudy_Pixel");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->number_cloudy_pixel, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Number_Indetermined_Pixel");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->number_indetermined_pixel, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Optical_Thickness");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_optical_thickness, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Water_Path");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->cloud_water_path, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Top_Pressure");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name,&record->cloud_top_pressure, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Effective_Radius");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name,&record->cloud_effective_radius, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Phase_Optical_Properties");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->cloud_phase_optical_properties, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Multi_Layer_Flag");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->cloud_multi_layer_flag, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Fraction");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name,&record->cloud_fraction, start_1D, NULL, edges_1D, default_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Mask_5km");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->cloud_mask_5km, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Cloud_Mask_1km");
    edges_2D [1] = 2;
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->cloud_mask_1km, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Quality_Assurance_5km");
    edges_2D [1] = 10;
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->quality_assurance_5km, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"MYD06_Quality_Assurance_1km");
    edges_2D [1] = 5;
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->quality_assurance_1km, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
}

void print_caltrack_record_MODIS_MYD06( const caltrack_record_MODIS_MYD06 const * record ) {
    printf( "Latitude %f\n"          ,              record->lat );
    printf( "Longitude %f\n"         ,              record->lon );
    printf( "Time %e\n"              ,              record->time );
    printf( "Input index [%d,%d]\n"  ,              record->input_pixel_index[0], record->input_pixel_index[1] );
    printf( "Input File index %d\n"  ,              record->input_file_index);
    printf( "Input File %s\n"        ,              record->input_file );
    printf( "Brightness Temperature Band 29 %e\n",  record->brightness_temperature_band_29 );
    printf( "Brightness Temperature Band 31 %e\n",  record->brightness_temperature_band_31 );
    printf( "Brightness Temperature Band 32 %e\n",  record->brightness_temperature_band_32 );
    printf( "Brightness Temperature Band 33 %e\n",  record->brightness_temperature_band_33 );
    printf( "Brightness Temperature Band 34 %e\n",  record->brightness_temperature_band_34 );
    printf( "Brightness Temperature Band 35 %e\n",  record->brightness_temperature_band_35 );
    printf( "Brightness Temperature Band 36 %e\n",  record->brightness_temperature_band_36 );
    printf( "Number Clear Pixel %d\n",              record->number_clear_pixel );
    printf( "Number Cloudy Pixel %d\n",             record->number_cloudy_pixel );
    printf( "Number Indetermined Pixel %d\n",       record->number_indetermined_pixel );
    printf( "Cloud Optical Thickness %e\n",         record->cloud_optical_thickness );
    printf( "Cloud Water Path %e\n",                record->cloud_water_path );
    printf( "Cloud Top Pressure %e\n",              record->cloud_top_pressure );
    printf( "Cloud Effective Radius %e\n",          record->cloud_effective_radius );
    printf( "Cloud Phase Optical Properties %d\n",  record->cloud_phase_optical_properties );
    printf( "Cloud Multi Layer Flag %d\n",          record->cloud_multi_layer_flag );
    printf( "Cloud Fraction %e\n",                  record->cloud_fraction );
    printf( "Cloud Mask 5km %d\n",                  record->cloud_mask_5km );
    printf( "Cloud Mask 1km %d %d\n",               record->cloud_mask_1km[0], record->cloud_mask_1km[1] );
    printf( "QA 5km %d %d %d %d %d %d %d %d %d %d\n",   record->quality_assurance_5km[0],
                                                        record->quality_assurance_5km[1],
                                                        record->quality_assurance_5km[2],
                                                        record->quality_assurance_5km[3],
                                                        record->quality_assurance_5km[4],
                                                        record->quality_assurance_5km[5],
                                                        record->quality_assurance_5km[6],
                                                        record->quality_assurance_5km[7],
                                                        record->quality_assurance_5km[8],
                                                        record->quality_assurance_5km[9] );
    printf( "QA 1km %d %d %d %d %d\n",                  record->quality_assurance_1km[0],
                                                        record->quality_assurance_1km[1],
                                                        record->quality_assurance_1km[2],
                                                        record->quality_assurance_1km[3],
                                                        record->quality_assurance_1km[4] );

}
