#include "caltrack_reader_parasol_l1.h"

int read_caltrack_record_PARASOL_L1( const char* filename, const int i_rec, caltrack_record_PARASOL_L1 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,-1};
    /* name of an SDS */
    char sds_name[MAX_NC_NAME];
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        return FAIL;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* grid index */
    strcpy(sds_name,"P3L1_Input_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->input_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Number of available viewing directions */
    strcpy(sds_name,"P3L1_Number_Available_Viewing_Direction");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->nb_available_viewing_direction, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Pixel Quality Index */
    strcpy(sds_name,"P3L1_Pixel_Quality_Index");
    edges_2D[1]=16; // 16 values to read
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->pixel_quality_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Solar Azimuth Angle */
    strcpy(sds_name,"P3L1_Solar_Azimuth_Angle");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->solar_azimuth_angle, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Solar Zenith Angle */
    strcpy(sds_name,"P3L1_Solar_Zenith_Angle");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->solar_zenith_angle, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* view zenith angle filter#8 */
    strcpy(sds_name,"P3L1_View_Zenith_Angle_670P2");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->view_zenith_angle_670p2, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Relative Azimuth Angle filter#8 */
    strcpy(sds_name,"P3L1_Relative_Azimuth_Angle_670P2");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->relative_azimuth_angle_670p2, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Delta Theta_v Cos_phi */
    strcpy(sds_name,"P3L1_Delta_Thetav_Cosphi");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->delta_thetav_cosphi, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Delta Theta_v Sin_phi */
    strcpy(sds_name,"P3L1_Delta_Thetav_Sinphi");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->delta_thetav_sinphi, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 443NP */
    strcpy(sds_name,"P3L1_Normalized_Radiance_443NP");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_443np, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 1020NP */
    strcpy(sds_name,"P3L1_Normalized_Radiance_1020NP");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_1020np, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 565NP */
    strcpy(sds_name,"P3L1_Normalized_Radiance_565NP");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_565np, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 763NP */
    strcpy(sds_name,"P3L1_Normalized_Radiance_763NP");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_763np, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 765NP */
    strcpy(sds_name,"P3L1_Normalized_Radiance_765NP");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_765np, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 910NP */
    strcpy(sds_name,"P3L1_Normalized_Radiance_910NP");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_910np, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 490P */
    strcpy(sds_name,"P3L1_Normalized_Radiance_490P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_490p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 670P */
    strcpy(sds_name,"P3L1_Normalized_Radiance_670P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_670p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Normalized Radiance channel 865P */
    strcpy(sds_name,"P3L1_Normalized_Radiance_865P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->normalized_radiance_865p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Second (Q) Stokes Vector 490P */
    strcpy(sds_name,"P3L1_Q_Stokes_490P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name,record->q_stokes_490p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Second (Q) Stokes Vector 670P */
    strcpy(sds_name,"P3L1_Q_Stokes_670P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->q_stokes_670p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Second (Q) Stokes Vector 865P */
    strcpy(sds_name,"P3L1_Q_Stokes_865P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->q_stokes_865p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Third (U) Stokes Vector 490P */
    strcpy(sds_name,"P3L1_U_Stokes_490P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->u_stokes_490p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Third (U) Stokes Vector 670P */
    strcpy(sds_name,"P3L1_U_Stokes_670P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->u_stokes_670p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* Third (U) Stokes Vector 865P */
    strcpy(sds_name,"P3L1_U_Stokes_865P");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, record->u_stokes_865p, start_2D, NULL, edges_2D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
};

void print_caltrack_record_PARASOL_L1 ( const caltrack_record_PARASOL_L1 const * record ) {
    printf( "Latitude %f\n"          ,  record->lat );
    printf( "Longitude %f\n"         ,  record->lon );
    printf( "Time %e\n"              ,  record->time );
    printf( "Grid index [%d,%d]\n", record->input_index[0], record->input_index[1] );
    printf( "Number of available viewing directions %u\n", record->nb_available_viewing_direction );
    printf( "Solar Azimuth Angle %e\n", record->solar_azimuth_angle );
    printf ( "Pixel Quality Index (hexadecimal) %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x\n",
            record->pixel_quality_index[0],  record->pixel_quality_index[1],  record->pixel_quality_index[2],  record->pixel_quality_index[3],
            record->pixel_quality_index[4],  record->pixel_quality_index[5],  record->pixel_quality_index[6],  record->pixel_quality_index[7],
            record->pixel_quality_index[8],  record->pixel_quality_index[9],  record->pixel_quality_index[10], record->pixel_quality_index[11],
            record->pixel_quality_index[12], record->pixel_quality_index[13], record->pixel_quality_index[14], record->pixel_quality_index[15]);
    printf ( "Solar Zenith Angle %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->solar_zenith_angle[0],  record->solar_zenith_angle[1],  record->solar_zenith_angle[2],  record->solar_zenith_angle[3],
            record->solar_zenith_angle[4],  record->solar_zenith_angle[5],  record->solar_zenith_angle[6],  record->solar_zenith_angle[7],
            record->solar_zenith_angle[8],  record->solar_zenith_angle[9],  record->solar_zenith_angle[10], record->solar_zenith_angle[11],
            record->solar_zenith_angle[12], record->solar_zenith_angle[13], record->solar_zenith_angle[14], record->solar_zenith_angle[15]);
    printf ( "View Zenith Angle filter#8 %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->view_zenith_angle_670p2[0],  record->view_zenith_angle_670p2[1],  record->view_zenith_angle_670p2[2],  record->view_zenith_angle_670p2[3],
            record->view_zenith_angle_670p2[4],  record->view_zenith_angle_670p2[5],  record->view_zenith_angle_670p2[6],  record->view_zenith_angle_670p2[7],
            record->view_zenith_angle_670p2[8],  record->view_zenith_angle_670p2[9],  record->view_zenith_angle_670p2[10], record->view_zenith_angle_670p2[11],
            record->view_zenith_angle_670p2[12], record->view_zenith_angle_670p2[13], record->view_zenith_angle_670p2[14], record->view_zenith_angle_670p2[15]);
    printf ( "Relative Azimuth Angle filter#8 %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->relative_azimuth_angle_670p2[0],  record->relative_azimuth_angle_670p2[1],  record->relative_azimuth_angle_670p2[2],  record->relative_azimuth_angle_670p2[3],
            record->relative_azimuth_angle_670p2[4],  record->relative_azimuth_angle_670p2[5],  record->relative_azimuth_angle_670p2[6],  record->relative_azimuth_angle_670p2[7],
            record->relative_azimuth_angle_670p2[8],  record->relative_azimuth_angle_670p2[9],  record->relative_azimuth_angle_670p2[10], record->relative_azimuth_angle_670p2[11],
            record->relative_azimuth_angle_670p2[12], record->relative_azimuth_angle_670p2[13], record->relative_azimuth_angle_670p2[14], record->relative_azimuth_angle_670p2[15]);
    printf ( "Delta Theta_v Cos_phi %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->delta_thetav_cosphi[0],  record->delta_thetav_cosphi[1],  record->delta_thetav_cosphi[2],  record->delta_thetav_cosphi[3],
            record->delta_thetav_cosphi[4],  record->delta_thetav_cosphi[5],  record->delta_thetav_cosphi[6],  record->delta_thetav_cosphi[7],
            record->delta_thetav_cosphi[8],  record->delta_thetav_cosphi[9],  record->delta_thetav_cosphi[10], record->delta_thetav_cosphi[11],
            record->delta_thetav_cosphi[12], record->delta_thetav_cosphi[13], record->delta_thetav_cosphi[14], record->delta_thetav_cosphi[15]);
    printf ( "Delta Theta_v Sin_phi %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->delta_thetav_sinphi[0],  record->delta_thetav_sinphi[1],  record->delta_thetav_sinphi[2],  record->delta_thetav_sinphi[3],
            record->delta_thetav_sinphi[4],  record->delta_thetav_sinphi[5],  record->delta_thetav_sinphi[6],  record->delta_thetav_sinphi[7],
            record->delta_thetav_sinphi[8],  record->delta_thetav_sinphi[9],  record->delta_thetav_sinphi[10], record->delta_thetav_sinphi[11],
            record->delta_thetav_sinphi[12], record->delta_thetav_sinphi[13], record->delta_thetav_sinphi[14], record->delta_thetav_sinphi[15]);
    printf ( "Normalized Radiance 443NP %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_443np[0],  record->normalized_radiance_443np[1],  record->normalized_radiance_443np[2],  record->normalized_radiance_443np[3],
            record->normalized_radiance_443np[4],  record->normalized_radiance_443np[5],  record->normalized_radiance_443np[6],  record->normalized_radiance_443np[7],
            record->normalized_radiance_443np[8],  record->normalized_radiance_443np[9],  record->normalized_radiance_443np[10], record->normalized_radiance_443np[11],
            record->normalized_radiance_443np[12], record->normalized_radiance_443np[13], record->normalized_radiance_443np[14], record->normalized_radiance_443np[15]);
    printf ( "Normalized Radiance 490P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_490p[0],  record->normalized_radiance_490p[1],  record->normalized_radiance_490p[2],  record->normalized_radiance_490p[3],
            record->normalized_radiance_490p[4],  record->normalized_radiance_490p[5],  record->normalized_radiance_490p[6],  record->normalized_radiance_490p[7],
            record->normalized_radiance_490p[8],  record->normalized_radiance_490p[9],  record->normalized_radiance_490p[10], record->normalized_radiance_490p[11],
            record->normalized_radiance_490p[12], record->normalized_radiance_490p[13], record->normalized_radiance_490p[14], record->normalized_radiance_490p[15]);
    printf ( "Normalized Radiance 1020NP %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_1020np[0],  record->normalized_radiance_1020np[1],  record->normalized_radiance_1020np[2],  record->normalized_radiance_1020np[3],
            record->normalized_radiance_1020np[4],  record->normalized_radiance_1020np[5],  record->normalized_radiance_1020np[6],  record->normalized_radiance_1020np[7],
            record->normalized_radiance_1020np[8],  record->normalized_radiance_1020np[9],  record->normalized_radiance_1020np[10], record->normalized_radiance_1020np[11],
            record->normalized_radiance_1020np[12], record->normalized_radiance_1020np[13], record->normalized_radiance_1020np[14], record->normalized_radiance_1020np[15]);
    printf ( "Normalized Radiance 565NP %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_565np[0],  record->normalized_radiance_565np[1],  record->normalized_radiance_565np[2],  record->normalized_radiance_565np[3],
            record->normalized_radiance_565np[4],  record->normalized_radiance_565np[5],  record->normalized_radiance_565np[6],  record->normalized_radiance_565np[7],
            record->normalized_radiance_565np[8],  record->normalized_radiance_565np[9],  record->normalized_radiance_565np[10], record->normalized_radiance_565np[11],
            record->normalized_radiance_565np[12], record->normalized_radiance_565np[13], record->normalized_radiance_565np[14], record->normalized_radiance_565np[15]);
    printf ( "Normalized Radiance 670P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_670p[0],  record->normalized_radiance_670p[1],  record->normalized_radiance_670p[2],  record->normalized_radiance_670p[3],
            record->normalized_radiance_670p[4],  record->normalized_radiance_670p[5],  record->normalized_radiance_670p[6],  record->normalized_radiance_670p[7],
            record->normalized_radiance_670p[8],  record->normalized_radiance_670p[9],  record->normalized_radiance_670p[10], record->normalized_radiance_670p[11],
            record->normalized_radiance_670p[12], record->normalized_radiance_670p[13], record->normalized_radiance_670p[14], record->normalized_radiance_670p[15]);
    printf ( "Normalized Radiance 763NP %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_763np[0],  record->normalized_radiance_763np[1],  record->normalized_radiance_763np[2],  record->normalized_radiance_763np[3],
            record->normalized_radiance_763np[4],  record->normalized_radiance_763np[5],  record->normalized_radiance_763np[6],  record->normalized_radiance_763np[7],
            record->normalized_radiance_763np[8],  record->normalized_radiance_763np[9],  record->normalized_radiance_763np[10], record->normalized_radiance_763np[11],
            record->normalized_radiance_763np[12], record->normalized_radiance_763np[13], record->normalized_radiance_763np[14], record->normalized_radiance_763np[15]);
    printf ( "Normalized Radiance 765NP %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_765np[0],  record->normalized_radiance_765np[1],  record->normalized_radiance_765np[2],  record->normalized_radiance_765np[3],
            record->normalized_radiance_765np[4],  record->normalized_radiance_765np[5],  record->normalized_radiance_765np[6],  record->normalized_radiance_765np[7],
            record->normalized_radiance_765np[8],  record->normalized_radiance_765np[9],  record->normalized_radiance_765np[10], record->normalized_radiance_765np[11],
            record->normalized_radiance_765np[12], record->normalized_radiance_765np[13], record->normalized_radiance_765np[14], record->normalized_radiance_765np[15]);
    printf ( "Normalized Radiance 865P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_865p[0],  record->normalized_radiance_865p[1],  record->normalized_radiance_865p[2],  record->normalized_radiance_865p[3],
            record->normalized_radiance_865p[4],  record->normalized_radiance_865p[5],  record->normalized_radiance_865p[6],  record->normalized_radiance_865p[7],
            record->normalized_radiance_865p[8],  record->normalized_radiance_865p[9],  record->normalized_radiance_865p[10], record->normalized_radiance_865p[11],
            record->normalized_radiance_865p[12], record->normalized_radiance_865p[13], record->normalized_radiance_865p[14], record->normalized_radiance_865p[15]);
    printf ( "Normalized Radiance 910NP %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->normalized_radiance_910np[0],  record->normalized_radiance_910np[1],  record->normalized_radiance_910np[2],  record->normalized_radiance_910np[3],
            record->normalized_radiance_910np[4],  record->normalized_radiance_910np[5],  record->normalized_radiance_910np[6],  record->normalized_radiance_910np[7],
            record->normalized_radiance_910np[8],  record->normalized_radiance_910np[9],  record->normalized_radiance_910np[10], record->normalized_radiance_910np[11],
            record->normalized_radiance_910np[12], record->normalized_radiance_910np[13], record->normalized_radiance_910np[14], record->normalized_radiance_910np[15]);
    printf ( "Second (Q) Stokes Vector 490P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->q_stokes_490p[0],  record->q_stokes_490p[1],  record->q_stokes_490p[2],  record->q_stokes_490p[3],
            record->q_stokes_490p[4],  record->q_stokes_490p[5],  record->q_stokes_490p[6],  record->q_stokes_490p[7],
            record->q_stokes_490p[8],  record->q_stokes_490p[9],  record->q_stokes_490p[10], record->q_stokes_490p[11],
            record->q_stokes_490p[12], record->q_stokes_490p[13], record->q_stokes_490p[14], record->q_stokes_490p[15]);
    printf ( "Second (Q) Stokes Vector 670P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->q_stokes_670p[0],  record->q_stokes_670p[1],  record->q_stokes_670p[2],  record->q_stokes_670p[3],
            record->q_stokes_670p[4],  record->q_stokes_670p[5],  record->q_stokes_670p[6],  record->q_stokes_670p[7],
            record->q_stokes_670p[8],  record->q_stokes_670p[9],  record->q_stokes_670p[10], record->q_stokes_670p[11],
            record->q_stokes_670p[12], record->q_stokes_670p[13], record->q_stokes_670p[14], record->q_stokes_670p[15]);
    printf ( "Second (Q) Stokes Vector 865P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->q_stokes_865p[0],  record->q_stokes_865p[1],  record->q_stokes_865p[2],  record->q_stokes_865p[3],
            record->q_stokes_865p[4],  record->q_stokes_865p[5],  record->q_stokes_865p[6],  record->q_stokes_865p[7],
            record->q_stokes_865p[8],  record->q_stokes_865p[9],  record->q_stokes_865p[10], record->q_stokes_865p[11],
            record->q_stokes_865p[12], record->q_stokes_865p[13], record->q_stokes_865p[14], record->q_stokes_865p[15]);
    printf ( "Third (U) Stokes Vector 490P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->u_stokes_490p[0],  record->u_stokes_490p[1],  record->u_stokes_490p[2],  record->u_stokes_490p[3],
            record->u_stokes_490p[4],  record->u_stokes_490p[5],  record->u_stokes_490p[6],  record->u_stokes_490p[7],
            record->u_stokes_490p[8],  record->u_stokes_490p[9],  record->u_stokes_490p[10], record->u_stokes_490p[11],
            record->u_stokes_490p[12], record->u_stokes_490p[13], record->u_stokes_490p[14], record->u_stokes_490p[15]);
    printf ( "Third (U) Stokes Vector 670P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->u_stokes_670p[0],  record->u_stokes_670p[1],  record->u_stokes_670p[2],  record->u_stokes_670p[3],
            record->u_stokes_670p[4],  record->u_stokes_670p[5],  record->u_stokes_670p[6],  record->u_stokes_670p[7],
            record->u_stokes_670p[8],  record->u_stokes_670p[9],  record->u_stokes_670p[10], record->u_stokes_670p[11],
            record->u_stokes_670p[12], record->u_stokes_670p[13], record->u_stokes_670p[14], record->u_stokes_670p[15]);
    printf ( "Third (U) Stokes Vector 865P %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e %e\n",
            record->u_stokes_865p[0],  record->u_stokes_865p[1],  record->u_stokes_865p[2],  record->u_stokes_865p[3],
            record->u_stokes_865p[4],  record->u_stokes_865p[5],  record->u_stokes_865p[6],  record->u_stokes_865p[7],
            record->u_stokes_865p[8],  record->u_stokes_865p[9],  record->u_stokes_865p[10], record->u_stokes_865p[11],
            record->u_stokes_865p[12], record->u_stokes_865p[13], record->u_stokes_865p[14], record->u_stokes_865p[15]);
}
