#include "caltrack_reader_parasol_ls2.h"

int read_caltrack_record_PARASOL_LS2( const char* filename, const int i_rec, caltrack_record_PARASOL_LS2 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,-1};
    /* name of an SDS */
    char sds_name[MAX_NC_NAME];
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        return FAIL;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* grid index */
    strcpy(sds_name,"P3L2TLGC_Input_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->input_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Land_Water_Indicator");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->land_water_indicator, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Pixel_Confidence_Data");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->pixel_confidence_data, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Aerosol_OD_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_od_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Aerosol_Model_Refractive_Index");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_model_refractive_index, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Aerosol_Model_Angstrom_Coefficient");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_model_angstrom_coefficient, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Aerosol_Index");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_index, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Fixed_Model_OD_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->fixed_model_od_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Quality_Index_Viewing_Geometry");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->quality_index_viewing_geometry, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TLGC_Quality_Index_Polarized_Reflectance_Fit");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->quality_index_polarized_reflectance_fit, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
};

void print_caltrack_record_PARASOL_LS2 ( const caltrack_record_PARASOL_LS2 const * record ) {
    printf( "Latitude %f\n",                                record->lat );
    printf( "Longitude %f\n",                               record->lon );
    printf( "Time %e\n",                                    record->time );
    printf( "Grid index [%d,%d]\n",                         record->input_index[0], record->input_index[1] );
    printf( "Land Water Indicator %u\n",                    record->land_water_indicator );
    printf( "Pixel Confidence Data %x\n",                   record->pixel_confidence_data );
    printf( "Aerosol Optical Depth 865 nm %e\n",            record->aerosol_od_865 );
    printf( "Aerosol Model Refractive Index %e\n",          record->aerosol_model_refractive_index );
    printf( "Aerosol Model Angstrm Coefficient %e\n",      record->aerosol_model_angstrom_coefficient );
    printf( "Aerosol Index %e\n",                           record->aerosol_index );
    printf( "Fixed Model Optical Depth 865nm %e\n",         record->fixed_model_od_865 );
    printf( "Quality Index Viewing Geometry %e\n",          record->quality_index_viewing_geometry );
    printf( "Quality Index Polarized Reflectance Fit %e\n", record->quality_index_polarized_reflectance_fit );
}
