/*
*  C Implementation: caltrack_reader_parasol_oc2
*
* Description: 
*
*
* Author: Nicolas PASCAL <nicolas.pascal@icare.univ-lille1.fr>, (C) 2008
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include "caltrack_reader_parasol_oc2.h"

int read_caltrack_record_PARASOL_OC2( const char* filename, const int i_rec, caltrack_record_PARASOL_OC2 *record ) {
    /* HDF functions return code */
    int32 status = FAIL;
    /* returned error code */
    int32 err     =  SUCCEED;
    /* HDF SD interface identifier */
    int32 sd_id = -1;
    /* read 1D variables */
    int32 start_1D[]= {i_rec};
    int32 edges_1D[]= {1};
    /* read 2D variables */
    int32 start_2D[]= {i_rec,0};
    int32 edges_2D[]= {1,NB_PARASOL_DIRECTION};
    /* name of an SDS */
    char sds_name[MAX_NC_NAME];
    /* generic pointer to data buffer */
    void* p_data = NULL;

    /* open HDF file for reading */
    sd_id = open_hdf_file(filename);
    if ( sd_id == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to open file %s\n",__FILE__,__LINE__,filename);
        return FAIL;
    }

    strcpy(sds_name,"Latitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lat, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Longitude");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->lon, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"Time");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->time, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    /* grid index */
    strcpy(sds_name,"P3L2TOGC_Input_Index");
    edges_2D[1]=2; // 2 values along x axis
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, record->input_index, start_2D, NULL, edges_2D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }


    strcpy(sds_name,"P3L2TOGC_Pixel_Confidence_Data");
    p_data = read_sds_binary_data_by_id(sd_id, sds_name, &record->pixel_confidence_data, start_1D, NULL, edges_1D );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Fit_Quality");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->fit_quality, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Solar_Zenith_Angle");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->solar_zenith_angle, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Aerosol_OD_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_od_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Aerosol_OD_670");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_od_670, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Aerosol_Angstrom_Coefficient");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_angstrom_coefficient, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Single_Scattering_Albedo_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->single_scattering_albedo_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Aerosol_Scattering_Asymmetry_Factor");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_scattering_asymmetry_factor, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Aerosol_Index");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_index, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Aerosol_Effective_Radius");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->aerosol_effective_radius, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Fine_Mode_Effective_Radius");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->fine_mode_effective_radius, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Large_Mode_Effective_Radius");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->large_mode_effective_radius, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Fine_Mode_OD_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->fine_mode_od_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Fine_Mode_OD_670");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->fine_mode_od_670, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Fine_Mode_Angstrom_Exponent");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->fine_mode_angstrom_exponent, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Spherical_Coarse_Mode_OD_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->spherical_coarse_mode_od_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Non_Spherical_Coarse_Mode_OD_865");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, (void*)&record->non_spherical_coarse_mode_od_865, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Relative_Contrib_Non_Spherical_Part_Coarse_Mode");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->relative_contrib_non_spherical_part_coarse_mode, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Refractive_Index_Fine_Mode");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->refractive_index_fine_mode, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Refractive_Index_Coarse_Mode");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->refractive_index_coarse_mode, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Log_Backscattering_Coefficient_565");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->log_backscattering_coefficient_565, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

    strcpy(sds_name,"P3L2TOGC_Log_Backscattering_Coefficient_1020");
    p_data = read_custom_scale_sds_data_by_id(sd_id, sds_name, &record->log_backscattering_coefficient_1020, start_1D, NULL, edges_1D, parasol_scale_function );
    if ( p_data == NULL ) {
        fprintf(stderr,"%s[L%d] Fail to read %s\n",__FILE__,__LINE__,sds_name);
        err = FAIL;
        goto exit;
    }

exit:
    /* close HDF file */
    status = close_hdf_file(sd_id);
    if ( status == FAIL ) {
        fprintf(stderr,"%s[L%d] Fail to close file %s\n",__FILE__,__LINE__,filename);
        err = FAIL;
    }

    return err;
};

void print_caltrack_record_PARASOL_OC2 ( const caltrack_record_PARASOL_OC2 const * record ) {
    printf( "Latitude %f\n",                                                        record->lat );
    printf( "Longitude %f\n",                                                       record->lon );
    printf( "Time %e\n",                                                            record->time );
    printf( "Grid index [%d,%d]\n",                                                 record->input_index[0], record->input_index[1] );
    printf( "Pixel Confidence Data %u\n",                                           (unsigned int)record->pixel_confidence_data );
    printf( "Fit Quality %e\n",                                                     record->fit_quality );
    printf( "Solar Zenith Angle %e\n",                                              record->solar_zenith_angle );
    printf( "Aerosol Optical Depth 865nm %e\n",                                     record->aerosol_od_865 );
    printf( "Aerosol Optical Depth 670nm %e\n",                                     record->aerosol_od_670 );
    printf( "Aerosol Angstrm Coefficient %e\n",                                    record->aerosol_angstrom_coefficient );
    printf( "Single Scattering Albedo 865nm %e\n",                                  record->single_scattering_albedo_865 );
    printf( "Aerosol Scattering Asymmetry Factor %e\n",                             record->aerosol_scattering_asymmetry_factor );
    printf( "Aerosol Index %e\n",                                                   record->aerosol_index );
    printf( "Aerosol Effective Radius %e\n",                                        record->aerosol_effective_radius );
    printf( "Fine Mode Effective Radius %e\n",                                      record->fine_mode_effective_radius );
    printf( "Large Mode Effective Radius %e\n",                                     record->large_mode_effective_radius );
    printf( "Fine Mode Optical Depth 865nm %e\n",                                   record->fine_mode_od_865 );
    printf( "Fine Mode Optical Depth 670nm %e\n",                                   record->fine_mode_od_670 );
    printf( "Fine Mode Angstrom Exponent %e\n",                                     record->fine_mode_angstrom_exponent );
    printf( "Spherical Coarse Mode Optical Depth 865nm %e\n",                       record->spherical_coarse_mode_od_865 );
    printf( "Non Spherical Coarse Mode Optical Depth 865nm %e\n",                   record->non_spherical_coarse_mode_od_865 );
    printf( "Relative Contribution of Non Spherical Particles in Coarse Mode %e\n", record->relative_contrib_non_spherical_part_coarse_mode );
    printf( "Refractive Index Fine Mode %e\n",                                      record->refractive_index_fine_mode );
    printf( "Refractive Index Coarse Mode %e\n",                                    record->refractive_index_coarse_mode );
    printf( "Logarithm of the Backscattering Coefficient at 565nm %e\n",            record->log_backscattering_coefficient_565 );
    printf( "Logarithm of the Backscattering Coefficient at 1020nm %e\n",           record->log_backscattering_coefficient_1020 );
}
